#include "X3D_Cone.h"

namespace X3DTK {
namespace X3D {

Cone::Cone()
: X3DGeometry3DNode(), _bottomRadius(1.0f), _height(2.0f), _side(true), _bottom(true)  
{
  defineTypeName("Cone", "Geometry3D");
}

Cone::Cone(const SFFloat &bottomRadius, const SFFloat &height, const SFBool &side, const SFBool &bottom)
: X3DGeometry3DNode(), _bottomRadius(bottomRadius), _height(height), _side(side), _bottom(bottom)
{
  defineTypeName("Cone", "Geometry3D");
}

Cone::Cone(const Cone &C)
: X3DGeometry3DNode(C), _bottomRadius(C._bottomRadius), _height(C._height), _side(C._side), _bottom(C._bottom)
{
}

SFAbstractNode Cone::clone() const
{
  return new Cone(*this);
}

Cone::~Cone()
{
}

void Cone::setBottomRadius(const SFFloat &bottomRadius)
{
  _bottomRadius = bottomRadius;
}

void Cone::setHeight(const SFFloat &height)
{
  _height = height;
}

void Cone::setSide(const SFBool &side)
{
  _side = side;
}

void Cone::setBottom(const SFBool &bottom)
{
  _bottom = bottom;
}

void Cone::load(const X3DFileElement *element)
{
  int index;
  index = element->getIndexAttribute("bottomRadius");
  if (index != -1)
    _bottomRadius = element->getAttribute(index).toFloat(); 

  index = element->getIndexAttribute("height");
  if (index != -1)
    _height = element->getAttribute(index).toFloat(); 
  
  index = element->getIndexAttribute("side");
  if (index != -1)
    _side = (element->getAttribute(index).upper() == "TRUE"); 
  
  index = element->getIndexAttribute("bottom");
  if (index != -1)
    _bottom = (element->getAttribute(index).upper() == "TRUE");
}

SFString &Cone::write(SFString &output) const
{
  if (_bottomRadius != 1.0f)
    output += " bottomRadius=\"" + toSFString(_bottomRadius) + "\"";
  if (_height != 2.0f)  
    output += " height=\"" + toSFString(_height) + "\"";
  if (!_side)  
    output += " side=\"FALSE\"";
  if (!_bottom)  
    output += " bottom=\"FALSE\"";
      
  return output;
}

}
}
