#include "X3D_GLBuilderGeometry3DVisitor.h"
#include "X3D_Box.h"
#include "X3D_Cone.h"
#include "X3D_Cylinder.h"
#include "X3D_IndexedFaceSet.h"
#include "X3D_Sphere.h"
#include "GL_Box.h"
#include "GL_Cone.h"
#include "GL_Cylinder.h"
#include "GL_IndexedFaceSet.h"
#include "GL_Sphere.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

GLBuilderGeometry3DVisitor::GLBuilderGeometry3DVisitor()
: Geometry3DVisitor()
{
  // Enter functions.
  defineEnterFunction(&GLBuilderGeometry3DVisitor::enterBox);
  defineEnterFunction(&GLBuilderGeometry3DVisitor::enterCone);
  defineEnterFunction(&GLBuilderGeometry3DVisitor::enterCylinder);
  defineEnterFunction(&GLBuilderGeometry3DVisitor::enterIndexedFaceSet);
  defineEnterFunction(&GLBuilderGeometry3DVisitor::enterSphere);

  // StateVariables assignation.
  stateVariables = GraphTraversal::getInstanceOf<GLBuilderStateVariables>();
}

GLBuilderGeometry3DVisitor::~GLBuilderGeometry3DVisitor()
{
}

void GLBuilderGeometry3DVisitor::enterBox(Box *B) const
{
  GL::SFNode GB = stateVariables->getNode(B);
  if (GB == 0)
  {
    GB = new GL::Box();
    GB->setX3DReference(B);
    stateVariables->addCoupleNode(B, GB);
  }
  stateVariables->pushNode(GB);
}
  
void GLBuilderGeometry3DVisitor::enterCone(Cone *C) const
{
  GL::SFNode GC = stateVariables->getNode(C);
  if (GC == 0)
  {
    GC = new GL::Cone();
    GC->setX3DReference(C);
    stateVariables->addCoupleNode(C, GC);
  }
  stateVariables->pushNode(GC);
}

void GLBuilderGeometry3DVisitor::enterCylinder(Cylinder *C) const
{
  GL::SFNode GC = stateVariables->getNode(C);
  if (GC == 0)
  {
    GC = new GL::Cylinder();
    GC->setX3DReference(C);
    stateVariables->addCoupleNode(C, GC);
  }
  stateVariables->pushNode(GC);
}

void GLBuilderGeometry3DVisitor::enterIndexedFaceSet(IndexedFaceSet *I) const
{
  GL::SFNode GI = stateVariables->getNode(I);
  if (GI == 0)
  {
    GI = new GL::IndexedFaceSet();
    GI->setX3DReference(I);
    stateVariables->addCoupleNode(I, GI);
  }
  stateVariables->pushNode(GI);

}
void GLBuilderGeometry3DVisitor::enterSphere(Sphere *S) const
{
  GL::SFNode GS = stateVariables->getNode(S);
  if (GS == 0)
  {
    GS = new GL::Sphere();
    GS->setX3DReference(S);
    stateVariables->addCoupleNode(S, GS);
  }
  stateVariables->pushNode(GS);
}

}
}
