#include "X3D_MeshBuilderStateVariables.h"
#include "X3D_X3DAppearanceNode.h"
#include "X3D_Shape.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

MeshBuilderStateVariables::MeshBuilderStateVariables()
: StateVariables(), _root(0), _childrenVisit(true), _cloner(new Cloner())
{
}

MeshBuilderStateVariables::~MeshBuilderStateVariables()
{
  delete _cloner;
}

void MeshBuilderStateVariables::init()
{
  _root = 0;
  _nodeStack.push_front(0); 
  _childrenVisit = true;
  _cloner->init();
}

void MeshBuilderStateVariables::finish()
{
  _nodeStack.clear();
  _nodeCoupleMap.clear();
  _cloner->finish();
}

void MeshBuilderStateVariables::enableChildrenVisit()
{
  _childrenVisit = true;
}

void MeshBuilderStateVariables::disableChildrenVisit()
{
  _childrenVisit = false;
}


X3DNode *MeshBuilderStateVariables::clone(X3DNode *N)
{
  return static_cast<X3DNode *>(_cloner->clone(N, false));
}
  
void MeshBuilderStateVariables::addCoupleNode(SFNode N, MESH::SFNode MN)
{
  _nodeCoupleMap[N] = MN;  
  
  // init root node.
  if (_root == 0)
    _root = MN;
}

void MeshBuilderStateVariables::pushNode(MESH::SFNode N)
{
  MESH::SFNode T = getTop();
  if (T != 0)
  {
    T->addChild(N);
  }
  _nodeStack.push_front(N);
}

void MeshBuilderStateVariables::popNode()
{
  _nodeStack.pop_front();
}

MESH::SFNode MeshBuilderStateVariables::getNode(SFNode N) const
{
  map<SFNode, MESH::SFNode>::const_iterator it = _nodeCoupleMap.find(N);
  if (it != _nodeCoupleMap.end())
    return (*it).second;
    
  return 0;  
}

}
}
