#include "X3D_SharedCoordComputerRenderingVisitor.h"
#include "X3D_Coordinate.h"
#include "X3D_IndexedFaceSet.h"

#include <iostream>
#include <list>

using namespace std;

namespace X3DTK {
namespace X3D {

SharedCoordComputerRenderingVisitor::SharedCoordComputerRenderingVisitor()
: RenderingVisitor()
{
  // Enter functions.
  defineEnterFunction(&SharedCoordComputerRenderingVisitor::enterCoordinate);
  
  // StateVariables assignation.
  stateVariables = GraphTraversal::getInstanceOf<SharedCoordComputerStateVariables>();
}

SharedCoordComputerRenderingVisitor::~SharedCoordComputerRenderingVisitor()
{
}

void SharedCoordComputerRenderingVisitor::enterCoordinate(Coordinate *C) const
{
  if (!stateVariables->getNode(C))
  {
    // Checking the number of IndexedFaceSet parents.
    MFAbstractNode parentList = C->getParentList();
    list<IndexedFaceSet *> ifslist;
    for (MFAbstractNode::const_iterator it = parentList.begin(); it != parentList.end(); ++it)
      if (dynamic_cast<IndexedFaceSet *>(*it) != 0) 
      {
        ifslist.push_back(static_cast<IndexedFaceSet *>(*it));
      } 

    if (ifslist.size() > 1)
    {
      map<IndexedFaceSet *, std::pair<bool, list<SFAbstractNode> > > &ifsmap = stateVariables->find(ifslist);
      
      // Removing the parents of C.
      for (map<IndexedFaceSet *, std::pair<bool, list<SFAbstractNode> > >::const_iterator it = ifsmap.begin(); it != ifsmap.end(); ++it)
        (*it).first->removeChild(C);
      
      // processing the IndexedFaceSet.
      bool cloned = false;
      unsigned int n = 0;
      map<IndexedFaceSet *, std::pair<bool, list<SFAbstractNode> > >::iterator it = ifsmap.begin();
      while (it != ifsmap.end())
      {
        if (!(*it).second.first)
        {
          SFAbstractNode NC;
          if (!cloned)
          {
            NC = C;
            cloned = true;
          }  
          else
          {
            NC = C->clone();  
            NC->setName("");
            ++n;
          }  
        
          (*it).first->setChild(NC);
          
          map<IndexedFaceSet *, std::pair<bool, list<SFAbstractNode> > >::iterator itF = it;
          ++itF;
          while (itF != ifsmap.end())
          {
            if (!(*itF).second.first)
            {
              // testing compatibility between (*it).first and (*itF).first
              list<SFAbstractNode> l = (*it).second.second;
              list<SFAbstractNode> lF = (*itF).second.second;
              
              if ((l.size() == 1) && (lF.size() == 1) && (l.front() == lF.front()))
              {
                //cout << "fusion " << (*it).first << " and " << (*itF).first << endl;
                (*itF).first->setChild(NC);
                (*itF).second.first = true;
              }
            }
            ++itF;
          }
        }  
        ++it;
      }
    }
    stateVariables->addNode(C);
  }
}

}
}
