#include "X3DAbstractNode.h"
#include "SFType.h"

#include <algorithm>
#include <iostream>

using namespace std;

namespace X3DTK {

X3DAbstractNode::X3DAbstractNode()
: _name(""), _type(0)
{    
  defineTypeName("X3DAbstractNode", "Abstract", "Abstract");
}

X3DAbstractNode::X3DAbstractNode(const X3DAbstractNode &N)
: _name(N._name), _type(N._type)
{
  addOneReference();
}

SFAbstractNode X3DAbstractNode::clone() const
{
  return 0;
}

X3DAbstractNode::~X3DAbstractNode()
{
  /// Removes the instance for all the parents
  MFAbstractNode parentList = _parentList;
  for (MFAbstractNode::const_iterator it = parentList.begin(); it != parentList.end(); ++it)
    (*it)->removeChild(this);

  //this method is recursive and removes one instance for all the parents of
  //the SFType.
  SFType::removeOneReference(_type);
}

void X3DAbstractNode::setName(const SFString &name)
{
  _name = name;
}

void X3DAbstractNode::addParent(const SFAbstractNode &N)
{
  if (N != 0)
    _parentList.push_back(N);
}

bool X3DAbstractNode::removeParent(const SFAbstractNode &N)
{
  if (N != 0)
  {
    MFAbstractNode::iterator res = find(_parentList.begin(), _parentList.end(), N);
    if (res != _parentList.end())
    {
      _parentList.erase(res);
      return true;
    }
  }
  return false;
}

MFAbstractNode X3DAbstractNode::getChildrenList() const
{
  return MFAbstractNode();
}

bool X3DAbstractNode::addChild(const SFAbstractNode &)
{
  cx3d << "warning : X3DAbstractNode::addChild must not be called, check that " << getTypeName() << " defines addChild!" << endl;
  return false;
}

bool X3DAbstractNode::setChild(const SFAbstractNode &)
{
  cx3d << "warning : X3DAbstractNode::setChild must not be called, check that " << getTypeName() << " defines setChild!" << endl;
  return false;
}

bool X3DAbstractNode::removeChild(const SFAbstractNode &)
{
  cx3d << "warning : X3DAbstractNode::removeChild must not be called, check that " << getTypeName() << " defines removeChild!" << endl;
  return false;
}

void X3DAbstractNode::addParentToChild(SFAbstractNode parent, SFAbstractNode child)
{
  if (child != 0)
    child->addParent(parent);
}

void X3DAbstractNode::removeParentFromChild(SFAbstractNode parent, SFAbstractNode child)
{
  if (child != 0)
    child->removeParent(parent);
}

}
